package pkg

import (
	"context"
	"errors"
	"net"
	"net/http"
	"time"

	"golang.org/x/text/language"
)

const (
	ContextUserKey   = "numerus-user"
	ContextCookieKey = "numerus-cookie"
	ContextConnKey   = "numerus-database"
	sessionCookie    = "numerus-session"
	defaultRole      = "guest"
)

type loginForm struct {
	locale   *Locale
	Errors   []error
	Email    *InputField
	Password *InputField
}

func newLoginForm(locale *Locale) *loginForm {
	return &loginForm{
		locale: locale,
		Email: &InputField{
			Name:     "email",
			Label:    pgettext("input", "Email", locale),
			Type:     "email",
			Required: true,
			Attributes: []*Attribute{
				{"autofocus", "autofocus"},
				{"autocomplete", "username"},
				{"autocapitalize", "none"},
			},
		},
		Password: &InputField{
			Name:     "password",
			Label:    pgettext("input", "Password", locale),
			Type:     "password",
			Required: true,
			Attributes: []*Attribute{
				{"autocomplete", "current-password"},
			},
		},
	}
}

func (form *loginForm) Parse(r *http.Request) error {
	err := r.ParseForm()
	if err != nil {
		return err
	}
	form.Email.FillValue(r)
	form.Password.FillValue(r)
	return nil
}

func (form *loginForm) Validate() bool {
	validator := newFormValidator()
	if validator.CheckRequiredInput(form.Email, gettext("Email can not be empty.", form.locale)) {
		validator.CheckValidEmailInput(form.Email, gettext("This value is not a valid email. It should be like name@domain.com.", form.locale))
	}
	validator.CheckRequiredInput(form.Password, gettext("Password can not be empty.", form.locale))
	return validator.AllOK()
}

func LoginHandler() http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		user := getUser(r)
		if user.LoggedIn {
			http.Redirect(w, r, "/", http.StatusSeeOther)
			return
		}
		locale := getLocale(r)
		form := newLoginForm(locale)
		if r.Method == "POST" {
			if err := form.Parse(r); err != nil {
				http.Error(w, err.Error(), http.StatusBadRequest)
				return
			}
			if form.Validate() {
				conn := getConn(r)
				cookie := conn.MustGetText(r.Context(), "", "select login($1, $2, $3)", form.Email.Value, form.Password.Value, remoteAddr(r))
				if cookie != "" {
					setSessionCookie(w, cookie)
					http.Redirect(w, r, "/", http.StatusSeeOther)
					return
				}
				form.Errors = append(form.Errors, errors.New(gettext("Invalid user or password.", locale)))
				w.WriteHeader(http.StatusUnauthorized)
			} else {
				w.WriteHeader(http.StatusUnprocessableEntity)
			}
		}
		mustRenderWebTemplate(w, r, "login.gohtml", form)
	})
}

func LogoutHandler() http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		conn := getConn(r)
		conn.MustExec(r.Context(), "select logout()")
		http.SetCookie(w, createSessionCookie("", -24*time.Hour))
		http.Redirect(w, r, "/login", http.StatusSeeOther)
	})
}

func remoteAddr(r *http.Request) string {
	address := r.Header.Get("X-Forwarded-For")
	if address == "" {
		address, _, _ = net.SplitHostPort(r.RemoteAddr)
	}
	return address
}

func setSessionCookie(w http.ResponseWriter, cookie string) {
	http.SetCookie(w, createSessionCookie(cookie, 8766*24*time.Hour))
}

func createSessionCookie(value string, duration time.Duration) *http.Cookie {
	return &http.Cookie{
		Name:     sessionCookie,
		Value:    value,
		Path:     "/",
		Expires:  time.Now().Add(duration),
		HttpOnly: true,
		SameSite: http.SameSiteLaxMode,
	}
}

type AppUser struct {
	Email    string
	LoggedIn bool
	Role     string
	Language language.Tag
}

func CheckLogin(db *Db, next http.Handler) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		var ctx = r.Context()
		if cookie, err := r.Cookie(sessionCookie); err == nil {
			ctx = context.WithValue(ctx, ContextCookieKey, cookie.Value)
		}

		conn := db.MustAcquire(ctx)
		defer conn.Release()
		ctx = context.WithValue(ctx, ContextConnKey, conn)

		user := &AppUser{
			Email:    "",
			LoggedIn: false,
			Role:     defaultRole,
		}
		row := conn.QueryRow(ctx, "select coalesce(email, ''), role, lang_tag from user_profile")
		var langTag string
		if err := row.Scan(&user.Email, &user.Role, &langTag); err != nil {
			panic(err)
		}
		user.LoggedIn = user.Email != ""
		user.Language, _ = language.Parse(langTag)
		ctx = context.WithValue(ctx, ContextUserKey, user)

		next.ServeHTTP(w, r.WithContext(ctx))
	})
}

func getUser(r *http.Request) *AppUser {
	return r.Context().Value(ContextUserKey).(*AppUser)
}

func getConn(r *http.Request) *Conn {
	return r.Context().Value(ContextConnKey).(*Conn)
}

func Authenticated(next http.Handler) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		user := getUser(r)
		if user.LoggedIn {
			next.ServeHTTP(w, r)
		} else {
			http.Redirect(w, r, "/login", http.StatusSeeOther)
		}
	})
}
